from json import dumps


class Sensor(object):

    def __init__(self, geojson_object=None, name=None, fields=None, location=None):
        """ Instantiates a Sensor object defining its structure
        either from a geojson object or from the related parameters.
        :param geojson_object: A dict loaded from a geojson object.
        :param name: The name of the sensor. Must only by used in
                     conjunction with fields.
        :param fields: A list of triples containing the name, data-type
                       and units of the sensor fields, as strings.
                       See GSN's documentation for a list of data-types.
        :param location: A triple giving the latitude, longitude and
                         altitude of the sensor, not mandatory.
        """
        assert geojson_object is None or (fields is None and name is None)

        if geojson_object:
            self.fields = [(f['name'], f['type'], f['unit'])
                           for f in geojson_object['properties']['fields']]
            self.name = geojson_object['properties']['vs_name']
            self.values = geojson_object['properties']['values']
            self.location = geojson_object['geometry']['coordinates']
        else:
            self.fields = fields
            self.name = name
            self.values = []
            self.location = location

    def to_geojson(self):

        r = {"type": "Feature",
             "properties": {"vs_name": self.name,
                            "values": self.values,
                            "fields": [{"name": f[0], "type": f[1], "unit": f[2]} for f in self.fields],
                            "stats": {},
                            "geographical": "",
                            "description": "Generated from python-gsn client"
                            },
             "geometry": {"type": "Point", "coordinates": self.location},
             "total_size": 1,
             "page_size": 1
             }
        return dumps(r)



from sanction import Client, transport_headers
from .sensor import Sensor
try:
    from urllib2 import HTTPError
except:
    from urllib.error import HTTPError


class API(object):
    """ Global Sensor Networks api client object
    """

    def __init__(self, service_url=None, client_id=None, client_secret=None, redirect_uri=None):
        """ Instantiates a GSN API client to authorize and authenticate a user
        :param service_url: The authorization endpoint provided by GSN
                            services.
        :param client_id: The client ID.
        :param client_secret: The client secret.
        """
        assert service_url is not None
        assert client_id is not None and client_secret is not None

        self.client = Client(token_endpoint="{}/oauth2/token".format(service_url),
                             resource_endpoint="{}/api".format(service_url),
                             client_id=client_id, client_secret=client_secret,
                             token_transport=transport_headers
                             )
        self.client.request_token(grant_type='client_credentials', redirect_uri=redirect_uri)

    def get_latest_values(self, vs_name=None):
        """ Query the API to get the latest values of a given virtual sensor.
        :param vs_name: The name of the virtual sensor.
        :returns: A Sensor object.
        """
        assert vs_name is not None

        data = self.client.request("/sensors/{}?latestValues=True".format(vs_name))
        return Sensor(geojson_object=data)

    def push_values(self, sensor_data=None):
        """ Push sensor data into GSN's API. The corresponding virtual sensor
         must be of type zeromq-push.
        :param sensor_data: A Sensor object containing the sensor values.
        :returns: The server response.
        """

        assert sensor_data is not None

        try:
            res = self.client.request("/sensors/{}/data".format(sensor_data.name),
                                      data=sensor_data.to_geojson().encode('utf_8'),
                                      headers={'Content-type': 'application/json'})
        except HTTPError as e:
            return e.readlines()
        return res
